package com.jalios.jcmsplugin.hourlyReport;

import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;

import com.jalios.jcms.Channel;
import com.jalios.jcms.analytics.AbstractStatisticAgent;
import com.jalios.jcms.analytics.AnalyticsManager;
import com.jalios.jcms.analytics.EventData;
import com.jalios.jcms.analytics.StatisticContext;
import com.jalios.jcms.analytics.metric.AbstractReportMetric;
import com.jalios.jcms.db.HibernateUtil;
import com.jalios.util.Util;

public class HourlyAgent extends AbstractStatisticAgent{

  private static Logger logger = Logger.getLogger(HourlyAgent.class);

  private HourlyReportMetric workMetric = null;
  private boolean ignoreWeekEnd = true;

  public HourlyAgent() {
    super();
  }

  public HourlyAgent(HourlyAgent agent) {
    super(agent);
  }

  @Override
  public AbstractStatisticAgent clone() {
    return new  HourlyAgent(this);
  }

  @Override
  public void startAnalyze(StatisticContext context) {
    workMetric = new HourlyReportMetric();
    ignoreWeekEnd= Channel.getChannel().getBooleanProperty("jcmsplugin.hourlyReport.HourlyAgent.ignoreWeekEnd", true);
  }

  @Override
  public void doProcess(EventData data, StatisticContext context) {
    //ignore event data which are not a HTTP request
    if(!data.isAccess()){
      if(logger.isTraceEnabled()){
        logger.trace("data is ignored because type '"+data.getType()+"' is not a compatible type.");
      }
      return;
    }
    //ignore bot access
    if(Util.toBoolean(data.getContext().get(AnalyticsManager.IS_BOT), false)){      
      if(logger.isTraceEnabled()){
        logger.trace("data is ignored because request is a bot request.");
      }
      return;
    }

    if(logger.isTraceEnabled()){
      logger.trace(this.toString() + " : "+data.toString());
    }

    //ignore anonymous user
    if(data.getActor() == null){
      return;
    }

    //ignore member if jcmsplugin.HourlyReport.isIgnored in context is set
    boolean isIgnored =Util.toBoolean(data.getContext().get("jcmsplugin.HourlyReport.isIgnored"), false);
    if(isIgnored){
      return;
    }

    //ignore week end data if ignoreWeekEnd is set
    Date startDate = new Date(data.getStartDate());    
    if(ignoreWeekEnd && isWeekEnd(startDate)){
      return;
    }
    
    workMetric.incrementCounter(startDate);
  }

  private boolean isWeekEnd(Date date){
    Calendar cal = Calendar.getInstance();
    cal.setTime(date);
    // this test is available for western countries but is not valid from some others countries (like Israel for example).
    int day = cal.get(Calendar.DAY_OF_WEEK);
    return day == Calendar.SATURDAY || day == Calendar.SUNDAY;
  }

  @Override
  public boolean endAnalyze(AbstractReportMetric reportMetric, StatisticContext context) {
    HourlyReportMetric hrReportMetric = (HourlyReportMetric) reportMetric;
    hrReportMetric.setValues(workMetric);
    HibernateUtil.evict(workMetric);
    workMetric = null;
    return true;
  }

  @Override
  public Class<? extends AbstractReportMetric> getReportMetricClass() {
    return HourlyReportMetric.class;
  }

}
